/* Scrutari - Copyright (c) 2008-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.servlets.admin.htmlpages;

import fr.exemole.sctserver.api.SctEngine;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;
import net.mapeadores.util.html.HA;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.localisation.LangList;
import net.mapeadores.util.text.Label;
import net.scrutari.data.MotcleData;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.api.LexieAccess;
import net.scrutari.db.api.codes.CodeList;
import net.scrutari.lexie.LexieOccurrencesList;
import net.scrutari.lexie.ScrutariLexieUnit;
import net.scrutari.lexie.ScrutariLexieUnitList;


/**
 *
 * @author Vincent Calame
 */
public class LexiesPage extends AdminHtmlPage {

    private final Lang libLang;
    private CodeList thesaurusCodeList;

    public LexiesPage(SctEngine engine, Lang libLang) {
        init(engine);
        this.libLang = libLang;
    }

    @Override
    public void handleResponse(HttpServletResponse response) throws IOException {
        DataAccess dataAccess = scrutariSession.getScrutariDB().openDataAccess();
        LexieAccess lexieAccess = scrutariSession.getScrutariDB().openLexieAccess();
        initResponse(response);
        start();
        printTools();
        printMessage();
        if (libLang == null) {
            printIndexResponse(response);
        } else {
            printLangResponse(response, dataAccess, lexieAccess);
        }

        end();
        lexieAccess.close();
        dataAccess.close();
    }

    private void printIndexResponse(HttpServletResponse response) throws IOException {
        LangList langList = scrutariSession.getScrutariDB().getStats().getAllMotcleLangList();
        int length = langList.getLangCount();
        for (int i = 0; i < length; i++) {
            String langString = langList.getLang(i).toString();
            startP();
            startA(HA.href("lexies_" + langString + ".html"));
            escape(langString);
            endA();
            space();
            escape('(');
            escape("ods : ");
            startA(HA.href("lexies_" + langString + ".ods"));
            escape("_all");
            endA();
            escape(')');
            endP();
        }
    }

    private void printLangResponse(HttpServletResponse response, DataAccess dataAccess, LexieAccess lexieAccess) throws IOException {
        thesaurusCodeList = dataAccess.getThesaurusCodeList();
        ScrutariLexieUnitList list = lexieAccess.getScrutariLexieUnitList(libLang);
        int count = list.getScrutariLexieUnitCount();
        startUl();
        for (int i = 0; i < count; i++) {
            ScrutariLexieUnit scrutariLexieUnit = list.getScrutariLexieUnit(i);
            if (scrutariLexieUnit.hasMotcleAndLexieLink()) {
                printLexieUnit(scrutariLexieUnit, dataAccess, lexieAccess);
            }
        }
        endUl();
    }

    private void printLexieUnit(ScrutariLexieUnit lexieUnit, DataAccess dataAccess, LexieAccess lexieAccess) throws IOException {
        startLi();
        startP();
        escape(lexieUnit.getCanonicalLexie());
        endP();
        startUl();
        long lexieId = lexieUnit.getLexieId();
        int length = thesaurusCodeList.getCodeCount();
        for (int i = 0; i < length; i++) {
            int codethesaurus = thesaurusCodeList.getCode(i);
            LexieOccurrencesList lexieOccurrencesList = lexieAccess.getLexieOccurrencesList(lexieId, codethesaurus);
            int codeCount = lexieOccurrencesList.getLexieOccurrencesCount();
            for (int j = 0; j < codeCount; j++) {
                int codemotcle = lexieOccurrencesList.getLexieOccurrences(j).getCode();
                MotcleData motcleData = dataAccess.getMotcleDataByCode(codemotcle);
                startLi();
                Label libelle = motcleData.getLabelByLang(libLang);
                escape(libelle.getLabelString());
                endLi();
            }
        }
        endUl();
        endLi();
    }

}
