/* Scrutari - Copyright (c) 2008-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.servlets.admin.htmlpages;

import fr.exemole.sctserver.api.SctEngine;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;
import net.mapeadores.util.html.HA;
import net.mapeadores.util.html.HTMLAttributes;
import net.mapeadores.util.localisation.LangList;
import net.mapeadores.util.text.Label;
import net.scrutari.data.BaseData;
import net.scrutari.data.MotcleData;
import net.scrutari.data.ThesaurusData;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.api.ScrutariDB;
import net.scrutari.db.tools.util.DataLabelUtils;


/**
 *
 * @author Vincent Calame
 */
public class MotsclesPage extends AdminHtmlPage {

    private final HTMLAttributes thesaurusCellAttributes = HA.classes("scrutari-list-Thesaurus");
    private final HTMLAttributes motcleTBodyAttributes = HA.style("display: none;");
    private final ScrutariDB scrutariDB;
    private final LangList motcleLangList;

    public MotsclesPage(SctEngine engine) {
        init(engine);
        scrutariDB = scrutariSession.getScrutariDB();
        addCssUrl("../css/listes.css");
        addJsUrl("../js/affichage.js");
        motcleLangList = scrutariDB.getStats().getAllMotcleLangList();
    }

    @Override
    public void handleResponse(HttpServletResponse response) throws IOException {
        DataAccess dataAccess = scrutariDB.openDataAccess();
        initResponse(response);
        start();
        printTools();
        printMessage();
        startP();
        escape("ods : ");
        startA(HA.href("motscles.ods"));
        escape("_all");
        endA();
        endP();
        startTable();
        startTHead(null);
        startRow();
        startCell();
        nonBreakableSpace();
        endCell();
        startCell();
        nonBreakableSpace();
        endCell();
        int length = motcleLangList.getLangCount();
        for (int i = 0; i < length; i++) {
            startCell();
            escape(motcleLangList.getLang(i));
            endCell();
        }
        endRow();
        endTHead();
        thesaurusCellAttributes.colspan(1 + length);
        HTMLAttributes baseCellAttr = HA.colspan(2 + length);
        int baseCount = dataAccess.getBaseCount();
        for (int i = 0; i < baseCount; i++) {
            BaseData baseData = dataAccess.getBaseDataByIndex(i);
            printBaseData(baseData, dataAccess, baseCellAttr);
        }
        endTable();
        end();
        dataAccess.close();
    }

    private void printBaseData(BaseData baseData, DataAccess dataAccess, HTMLAttributes baseCellAttr) {
        int thesaurusCount = baseData.getThesaurusCount();
        startTBody(null);
        startRow();
        startCell(baseCellAttr);
        escape(DataLabelUtils.checkLabelString(baseData, BaseData.INTITULE_LONG, langUi, "?"));
        space();
        startCode();
        escape('[');
        append(thesaurusCount);
        escape(']');
        endCode();
        endCell();
        endRow();
        endTBody();
        for (int i = 0; i < thesaurusCount; i++) {
            ThesaurusData thesaurusData = dataAccess.getThesaurusDataByCode(baseData.getCodeThesaurus(i));
            printThesaurusData(thesaurusData, dataAccess);
        }
    }

    private void printThesaurusData(ThesaurusData thesaurusData, DataAccess dataAccess) {
        int codethesaurus = thesaurusData.getCodeThesaurus();
        int motcleCount = thesaurusData.getMotcleCount();
        startTBody(HA.attr("onclick", "afficheRowGroup('tbody_" + codethesaurus + "');"));
        startRow();
        appendEmptyCell();
        startCell(thesaurusCellAttributes);
        escape(DataLabelUtils.checkLabelString(thesaurusData, langUi, "?"));
        space();
        startCode();
        escape('[');
        append(motcleCount);
        escape(']');
        endCode();
        endCell();
        endRow();
        endTBody();
        motcleTBodyAttributes.id("tbody_" + codethesaurus);
        startTBody(motcleTBodyAttributes);
        for (int i = 0; i < motcleCount; i++) {
            printMotcleData(dataAccess.getMotcleDataByCode(thesaurusData.getCodeMotcle(i)));
        }
        endTBody();

    }

    private void printMotcleData(MotcleData motcleData) {
        startRow();
        appendEmptyCell();
        appendEmptyCell();
        int length = motcleLangList.getLangCount();
        for (int i = 0; i < length; i++) {
            startCell();
            Label libelle = motcleData.getLabelByLang(motcleLangList.getLang(i));
            if (libelle != null) {
                escape(libelle.getLabelString());
            }
            endCell();
        }
        endRow();
    }

}
