/* Scrutari - Copyright (c) 2009-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.servlets.admin.htmlpages.utils;

import fr.exemole.sctserver.api.ScrutariSourceManager;
import fr.exemole.sctserver.api.collect.CollectResult;
import fr.exemole.sctserver.api.collect.ScrutariSource;
import fr.exemole.sctserver.api.collect.ScrutariSourceDef;
import fr.exemole.sctserver.api.collect.ScrutariSourceList;
import fr.exemole.sctserver.servlets.admin.htmlpages.AdminHtmlPage;
import fr.exemole.sctserver.tools.collect.CollectUtils;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.html.HA;
import net.mapeadores.util.html.HTMLAttributes;
import net.mapeadores.util.primitives.FuzzyDate;


/**
 *
 * @author Vincent Calame
 */
public final class SourcesPrint {

    private SourcesPrint() {
    }

    public static void printScrutariSourceList(AdminHtmlPage page) {
        ScrutariSourceManager scrutariSourceManager = page.getEngine().getScrutariSourceManager();
        ScrutariSourceList scrutariSourceList = scrutariSourceManager.getScrutariSourceList();
        int count = scrutariSourceList.getScrutariSourceCount();
        if (count == 0) {
            page.startP();
            page.startSpan(HA.classes("scrutari-admin-Info"));
            page.localize("_ info.sctconf.none_sources");
            page.endSpan();
            page.endP();
            return;
        }
        page.startTable(HA.classes("scrutari-admin-Sources"));
        for (int i = 0; i < count; i++) {
            ScrutariSource scrutariSource = scrutariSourceList.getScrutariSource(i);
            ScrutariSourceDef scrutariSourceDef = scrutariSource.getScrutariSourceDef();
            page.startRow();
            page.startCell();
            page.escape(scrutariSourceDef.getName());
            page.endCell();
            page.startCell();
            int urlCount = scrutariSourceDef.getUrlCount();
            for (int j = 0; j < urlCount; j++) {
                if (j > 0) {
                    page.appendBr();
                }
                String urlString = scrutariSourceDef.getUrl(j).toString();
                page.startA(HA.href(urlString));
                int idx = urlString.indexOf("://");
                if (idx != -1) {
                    urlString = urlString.substring(idx + 3);
                }
                page.escape(urlString);
                page.endA();
                page.space();
                page.startCode();
                page.escape('[');
                switch (scrutariSourceDef.getUrlType(j)) {
                    case ScrutariSourceDef.INFO_TYPE:
                        page.escape("info");
                        break;
                    case ScrutariSourceDef.SCRUTARIDATA_TYPE:
                        page.escape("scrutaridata");
                        break;
                }
                page.escape(']');
                page.endCode();
            }
            page.endCell();
            short state = scrutariSource.getState();
            page.startCell();
            page.startSpan(getSpanAttributes(state));
            page.localize(CollectUtils.getStateMessageKey(state));
            page.endSpan();
            page.endCell();
            page.startCell();
            FuzzyDate lastUpdate = scrutariSource.getLastUpdate();
            if (lastUpdate != null) {
                page.escape(lastUpdate.toISOString());
            }
            page.endCell();
            page.endRow();
        }
        page.endTable();
    }

    private static HTMLAttributes getSpanAttributes(short state) {
        switch (state) {
            case CollectResult.OK_STATE:
                return HA.classes("scrutari-admin-DoneMessage");
            case CollectResult.UNKNOWN_STATE:
            case CollectResult.INFO_NOCONNECTION_STATE:
            case CollectResult.INFO_BROKENURL_STATE:
            case CollectResult.INFO_DOWNLOADERROR_STATE:
            case CollectResult.INFO_NOTXML_STATE:
            case CollectResult.INFO_WRONGXML_STATE:
            case CollectResult.DATA_NOCONNECTION_STATE:
            case CollectResult.DATA_BROKENURL_STATE:
            case CollectResult.DATA_DOWNLOADERROR_STATE:
            case CollectResult.DATA_NOTXML_STATE:
                return HA.classes("scrutari-admin-ErrorMessage");
            default:
                throw new SwitchException("state = " + state);
        }
    }

}
