/* Scrutari - Copyright (c) 2009-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.servlets.webapp.htmlpages;

import fr.exemole.sctserver.api.log.SctLogConstants;
import fr.exemole.sctserver.servlets.ScrutariWebapp;
import java.io.IOException;
import java.util.List;
import javax.servlet.http.HttpServletResponse;
import net.mapeadores.util.html.HA;
import net.mapeadores.util.html.HTMLAttributes;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.localisation.LocalisationUtils;
import net.mapeadores.util.localisation.Message;
import net.mapeadores.util.logging.CategorizedMessageList;
import net.mapeadores.util.logging.MessageLog;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public class ListPage extends WebappHtmlPage {

    public ListPage(ScrutariWebapp scrutariWebapp, Lang langUi) {
        init(scrutariWebapp, langUi);
    }

    @Override
    public void handleResponse(HttpServletResponse response) throws IOException {
        initResponse(response);
        start();
        HTMLAttributes pAttr = HA.classes("");
        startP(pAttr.classes("scrutari-webapp-Title"));
        localize("_ info.webapp.updatedone");
        endP();
        MessageLog messageLog = scrutariWebapp.getInitMessageLog();
        int count = messageLog.getLogGroupCount();
        if (count > 0) {
            startP(pAttr.classes("scrutari-webapp-ErrorTitle"));
            localize("_ info.webapp.contexterror");
            endP();
            startUl();
            for (int i = 0; i < count; i++) {
                MessageLog.LogGroup logGroup = messageLog.getLogGroup(i);
                startLi();
                startP();
                startCode(null);
                escape('[');
                escape(logGroup.getURI());
                escape(']');
                endCode();
                endP();
                CategorizedMessageList messageList = logGroup.getList();
                int messageCount = messageList.getMessageCount();
                startUl();
                for (int j = 0; j < messageCount; j++) {
                    Message message = messageList.getMessage(j);
                    String category = messageList.getCategory(j);
                    startLi();
                    if (category.equals(SctLogConstants.CRITICAL_CATEGORY)) {
                        startP("scrutari-webapp-Error");
                    } else {
                        startP();
                    }
                    if (category.length() > 0) {
                        startCode();
                        escape('[');
                        escape(category);
                        escape(']');
                        endCode();
                        space();
                    }
                    localize(message);
                    endP();
                    endLi();
                }
                endUl();
                endLi();
            }
            endUl();

        }
        List<String> nameList = scrutariWebapp.getEngineNameList();
        int size = nameList.size();
        startP(pAttr.classes("scrutari-webapp-ListTitle"));
        if (size == 0) {
            localize("_ info.webapp.engine_none");
        } else if (size == 1) {
            localize("_ info.webapp.engine_one");
        } else {
            localize(LocalisationUtils.toMessage("_ info.webapp.engine_many", StringUtils.toString(size, langUi.toLocale())));
        }
        endP();
        if (size > 0) {
            startUl();
            for (String name : nameList) {
                startLi();
                escape(name);
                endLi();
            }
            endUl();
        }
        end();
    }

}
