/* ScrutariLib_DB - Copyright (c) 2014-2016 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.db.tools;

import net.mapeadores.util.logging.TimeLog;
import net.scrutari.datauri.ScrutariDataURIChecker;
import net.scrutari.db.api.AttributeKeyCoder;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.api.DataAccessProvider;
import net.scrutari.db.api.LexieAccess;
import net.scrutari.db.api.LexieAccessProvider;
import net.scrutari.db.api.ScrutariDB;
import net.scrutari.db.api.ScrutariDBName;
import net.scrutari.db.api.caches.DataCacheReaderFactory;
import net.scrutari.db.api.caches.LexieCacheReaderFactory;
import net.scrutari.db.api.caches.LexieCacheWriter;
import net.scrutari.db.api.stats.ScrutariDBStats;
import net.scrutari.db.tools.stats.ScrutariDBStatsBuilder;


/**
 *
 * @author Vincent Calame
 */
public class ScrutariDBBuilder {

    private ScrutariDBBuilder() {

    }

    public static ScrutariDB build(ScrutariDBName scrutariDBName, AttributeKeyCoder attributeKeyCoder, DataCacheReaderFactory dataCacheReaderFactory, LexieCacheWriter lexieCacheWriter, ScrutariDataURIChecker scrutariDataURIChecker, TimeLog timeLog) {
        DataAccessProvider dataAccessProvider = DataAccessProviderBuilder.build(dataCacheReaderFactory, scrutariDataURIChecker);
        timeLog.log("scrutaridb-dataccess_build");
        DataAccess dataAccess = dataAccessProvider.openDataAccess();
        LexieCacheReaderFactory lexieCacheReaderFactory = LexificationEngine.cacheLexification(dataAccess, attributeKeyCoder, lexieCacheWriter);
        timeLog.log("scrutaridb-lexification_cache");
        LexieAccessProvider lexieAccessProvider = LexieAccessProviderBuilder.build(lexieCacheReaderFactory);
        timeLog.log("scrutaridb-lexieaccess_build");
        LexieAccess lexieAccess = lexieAccessProvider.openLexieAccess();
        ScrutariDBStats stats = ScrutariDBStatsBuilder.build(dataAccess, lexieAccess);
        timeLog.log("scrutaridb-stats_build");
        dataAccess.close();
        lexieAccess.close();
        return new InternalScrutariDB(scrutariDBName, dataAccessProvider, lexieAccessProvider, attributeKeyCoder, stats);
    }

    public static ScrutariDB build(ScrutariDBName scrutariDBName, AttributeKeyCoder attributeKeyCoder, DataCacheReaderFactory dataCacheReaderFactory, LexieCacheReaderFactory lexieCacheReaderFactory, ScrutariDataURIChecker scrutariDataURIChecker, TimeLog timeLog) {
        DataAccessProvider dataAccessProvider = DataAccessProviderBuilder.build(dataCacheReaderFactory, scrutariDataURIChecker);
        timeLog.log("scrutaridb-dataccess_build");
        LexieAccessProvider lexieAccessProvider = LexieAccessProviderBuilder.build(lexieCacheReaderFactory);
        timeLog.log("scrutaridb-lexieaccess_build");
        DataAccess dataAccess = dataAccessProvider.openDataAccess();
        LexieAccess lexieAccess = lexieAccessProvider.openLexieAccess();
        ScrutariDBStats stats = ScrutariDBStatsBuilder.build(dataAccess, lexieAccess);
        timeLog.log("scrutaridb-stats_build");
        dataAccess.close();
        lexieAccess.close();
        return new InternalScrutariDB(scrutariDBName, dataAccessProvider, lexieAccessProvider, attributeKeyCoder, stats);
    }


    private static class InternalScrutariDB implements ScrutariDB {

        private final ScrutariDBName scrutariDBName;
        private final DataAccessProvider dataAccessProvider;
        private final LexieAccessProvider lexieAccessProvider;
        private final AttributeKeyCoder attributeKeyCoder;
        private final ScrutariDBStats stats;

        private InternalScrutariDB(ScrutariDBName scrutariDBName, DataAccessProvider dataAccessProvider, LexieAccessProvider lexieAccessProvider, AttributeKeyCoder attributeKeyCoder, ScrutariDBStats stats) {
            this.scrutariDBName = scrutariDBName;
            this.dataAccessProvider = dataAccessProvider;
            this.lexieAccessProvider = lexieAccessProvider;
            this.attributeKeyCoder = attributeKeyCoder;
            this.stats = stats;
        }

        @Override
        public ScrutariDBName getScrutariDBName() {
            return scrutariDBName;
        }

        @Override
        public DataAccess openDataAccess() {
            return dataAccessProvider.openDataAccess();
        }

        @Override
        public LexieAccess openLexieAccess() {
            return lexieAccessProvider.openLexieAccess();
        }

        @Override
        public AttributeKeyCoder getAttributeKeyCoder() {
            return attributeKeyCoder;
        }

        @Override
        public ScrutariDBStats getStats() {
            return stats;
        }

    }

}
