/* ScrutariLib_Data - Copyright (c) 2014-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.data;

import net.mapeadores.util.html.jsoup.TrustedHtmlFactories;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.text.CleanedString;
import net.mapeadores.util.text.Label;


/**
 *
 * @author Vincent Calame
 */
final class DataLabelBuilder {

    private DataLabelBuilder() {
    }

    static Label toLabel(Lang lang, String labelString) {
        return new DataLabel(lang, labelString);
    }

    static Label checkLabel(Label label) {
        if (label == null) {
            return null;
        }
        String labelString = label.getLabelString();
        String checkedString = TrustedHtmlFactories.TEXT.check(labelString);
        if (labelString.equals(checkedString)) {
            return label;
        } else {
            CleanedString cs = CleanedString.newInstance(checkedString);
            if (cs != null) {
                return new DataLabel(label.getLang(), cs.toString());
            } else {
                return null;
            }
        }
    }


    private static class DataLabel implements Label {

        private final String labelString;
        private final Lang lang;

        private DataLabel(Lang lang, String labelString) {
            this.lang = lang;
            this.labelString = labelString;
        }

        @Override
        public Lang getLang() {
            return lang;
        }

        @Override
        public String getLabelString() {
            return labelString;
        }

    }

}
