/* ScrutariLib_SearchEngine - Copyright (c) 2006-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.searchengine.tools.options;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.localisation.LangList;
import net.mapeadores.util.text.Label;
import net.mapeadores.util.text.LabelUtils;
import net.scrutari.data.CorpusData;
import net.scrutari.data.ThesaurusData;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.api.codes.CodeList;
import net.scrutari.db.api.stats.ScrutariDBStats;
import net.scrutari.db.tools.codes.CodesUtils;
import net.scrutari.searchengine.api.options.CategoryDef;
import net.scrutari.searchengine.api.options.SearchOptions;
import net.scrutari.searchengine.api.options.SearchOptionsDef;
import net.scrutari.searchengine.tools.EligibilityBuffer;


/**
 *
 * @author Vincent Calame
 */
public final class OptionsUtils {

    private OptionsUtils() {
    }

    public static CodeList getSelectionThesaurusCodeList(ScrutariDBStats scrutariDBStats, DataAccess dataAccess, CodeList corpusCodeList) {
        if (corpusCodeList == null) {
            return null;
        }
        Set<Integer> thesaurusSet = new LinkedHashSet<Integer>();
        int corpusCount = corpusCodeList.getCodeCount();
        CodeList thesaurusCodeList = dataAccess.getThesaurusCodeList();
        int thesaurusCount = thesaurusCodeList.getCodeCount();
        for (int i = 0; i < corpusCount; i++) {
            int codecorpus = corpusCodeList.getCode(i);
            for (int j = 0; j < thesaurusCount; j++) {
                int codethesaurus = thesaurusCodeList.getCode(j);
                if (scrutariDBStats.hasIndexation(codecorpus, codethesaurus)) {
                    thesaurusSet.add(codethesaurus);
                }
            }
        }
        return CodesUtils.toCodeList(thesaurusSet);
    }

    public static CodeList getFicheCodeList(DataAccess dataAccess, SearchOptions searchOptions) {
        EligibilityBuffer eligibilityBuffer = new EligibilityBuffer(dataAccess, searchOptions.getFicheEligibility());
        SearchOptionsDef searchOptionsDef = searchOptions.getSearchOptionsDef();
        Set<Lang> langSet = null;
        LangList langList = searchOptionsDef.getFilterLangList();
        int langCount = langList.getLangCount();
        if (langCount > 0) {
            langSet = new HashSet<Lang>();
            for (int i = 0; i < langCount; i++) {
                langSet.add(langList.getLang(i));
            }
        }
        CodeList corpusCodeList = searchOptions.getCorpusCodeList();
        if (corpusCodeList == null) {
            corpusCodeList = dataAccess.getCorpusCodeList();
        }
        List<Integer> list = new ArrayList<Integer>();
        int corpusCount = corpusCodeList.getCodeCount();
        for (int i = 0; i < corpusCount; i++) {
            int codecorpus = corpusCodeList.getCode(i);
            CorpusData corpusData = dataAccess.getCorpusDataByCode(codecorpus);
            int ficheCount = corpusData.getFicheCount();
            for (int j = 0; j < ficheCount; j++) {
                int codefiche = corpusData.getCodeFiche(j);
                boolean accept = true;
                if (langSet != null) {
                    Lang lang = dataAccess.getFicheLang(codefiche);
                    if (!langSet.contains(lang)) {
                        accept = false;
                    }
                }
                if ((accept) && (!eligibilityBuffer.acceptFiche(codefiche))) {
                    accept = false;
                }
                if (accept) {
                    list.add(codefiche);
                }
            }
        }
        return CodesUtils.toCodeList(list);
    }

    public static CodeList getMotcleCodeList(DataAccess dataAccess, SearchOptions searchOptions) {
        EligibilityBuffer eligibilityBuffer = new EligibilityBuffer(dataAccess, searchOptions.getFicheEligibility());
        CodeList thesaurusCodeList = searchOptions.getThesaurusCodeList();
        if (thesaurusCodeList == null) {
            thesaurusCodeList = dataAccess.getThesaurusCodeList();
        }
        List<Integer> list = new ArrayList<Integer>();
        int thesaurusCount = thesaurusCodeList.getCodeCount();
        for (int i = 0; i < thesaurusCount; i++) {
            int codethesaurus = thesaurusCodeList.getCode(i);
            ThesaurusData thesaurusData = dataAccess.getThesaurusDataByCode(codethesaurus);
            int motcleCount = thesaurusData.getMotcleCount();
            for (int j = 0; j < motcleCount; j++) {
                int codemotcle = thesaurusData.getCodeMotcle(j);
                if (eligibilityBuffer.acceptMotcle(codemotcle)) {
                    list.add(codemotcle);
                }
            }
        }
        return CodesUtils.toCodeList(list);
    }


    public static String checkLabelString(CategoryDef category, Lang lang, String defaultString) {
        Label label = LabelUtils.getLangPartCheckedLabel(category, lang);
        if (label == null) {
            label = LabelUtils.getFirstLabel(category);
        }
        if (label != null) {
            return label.getLabelString();
        } else {
            return defaultString;
        }
    }

}
