/* ScrutariLib_SearchEngine - Copyright (c) 2010-2014 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.searchengine.tools.result;

import net.mapeadores.util.localisation.Lang;
import net.scrutari.searchengine.api.result.InAlineaOccurrenceList;
import net.scrutari.searchengine.api.result.InLangOccurrenceList;
import net.scrutari.searchengine.api.result.SearchTokenOccurrence;
import net.scrutari.searchengine.api.result.SearchTokenOccurrenceList;
import net.scrutari.searchengine.api.result.SupermotcleSearchResult;
import net.scrutari.searchengine.api.result.SupermotcleSearchResultInfo;


/**
 *
 * @author Vincent Calame
 */
public class ResultUtils {

    public final static SupermotcleSearchResult EMPTY_SUPERMOTCLE_SEARCHRESULT = new EmptySupermotcleSearchResult();

    private ResultUtils() {
    }

    public static SearchTokenOccurrenceList toSearchTokenPositionList(SearchTokenOccurrence[] array) {
        return new ArraySearchTokenOccurrenceList(array);
    }

    public static InAlineaOccurrenceList toInAlineaOccurrenceList(int alineaInteger, SearchTokenOccurrence[] array) {
        return new ArrayInAlineaOccurrenceList(alineaInteger, array);
    }

    public static InLangOccurrenceList toInLangOccurrenceList(Lang lang, SearchTokenOccurrence[] array) {
        return new ArrayInLangOccurrenceList(lang, array);
    }


    private static class EmptySupermotcleSearchResult implements SupermotcleSearchResult {

        private EmptySupermotcleSearchResult() {

        }

        @Override
        public int getSupermotcleSearchResultInfoCount() {
            return 0;
        }

        @Override
        public SupermotcleSearchResultInfo getSupermotcleSearchResultInfo(int i) {
            throw new IndexOutOfBoundsException();
        }

    }


    private static class ArraySearchTokenOccurrenceList implements SearchTokenOccurrenceList {

        private final SearchTokenOccurrence[] array;

        private ArraySearchTokenOccurrenceList(SearchTokenOccurrence[] array) {
            this.array = array;
        }

        @Override
        public int getSearchTokenOccurrenceCount() {
            return array.length;
        }

        @Override
        public SearchTokenOccurrence getSearchTokenOccurrence(int i) {
            return array[i];
        }

    }


    private static class ArrayInAlineaOccurrenceList implements InAlineaOccurrenceList {

        private final int alineaInteger;
        private final SearchTokenOccurrence[] array;

        private ArrayInAlineaOccurrenceList(int alineaInteger, SearchTokenOccurrence[] array) {
            this.alineaInteger = alineaInteger;
            this.array = array;
        }

        @Override
        public int getAlineaInteger() {
            return alineaInteger;
        }

        @Override
        public int getSearchTokenOccurrenceCount() {
            return array.length;
        }

        @Override
        public SearchTokenOccurrence getSearchTokenOccurrence(int i) {
            return array[i];
        }

    }


    private static class ArrayInLangOccurrenceList implements InLangOccurrenceList {

        private final Lang lang;
        private final SearchTokenOccurrence[] array;

        private ArrayInLangOccurrenceList(Lang lang, SearchTokenOccurrence[] array) {
            this.lang = lang;
            this.array = array;
        }

        @Override
        public Lang getLang() {
            return lang;
        }

        @Override
        public int getSearchTokenOccurrenceCount() {
            return array.length;
        }

        @Override
        public SearchTokenOccurrence getSearchTokenOccurrence(int i) {
            return array[i];
        }

    }

}
