/* ScrutariLib_SearchEngine - Copyright (c) 2010-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.searchengine.tools.result;

import java.util.Comparator;
import net.scrutari.searchengine.api.result.SearchTokenOccurrence;
import net.scrutari.searchengine.api.result.SearchTokenOccurrenceList;


/**
 *
 * @author Vincent Calame
 */
public class SearchTokenOccurrenceListComparator implements Comparator<SearchTokenOccurrenceList> {

    public final static SearchTokenOccurrenceListComparator UNIQUE = new SearchTokenOccurrenceListComparator();

    private SearchTokenOccurrenceListComparator() {
    }

    public int compare(SearchTokenOccurrenceList list1, SearchTokenOccurrenceList list2) {
        int size1 = list1.getSearchTokenOccurrenceCount();
        int size2 = list2.getSearchTokenOccurrenceCount();
        if (size1 > size2) {
            return 1;
        }
        if (size1 < size2) {
            return -1;
        }
        int diff = 0;
        for (int i = 0; i < size1; i++) {
            SearchTokenOccurrence tokenOccurrence1 = list1.getSearchTokenOccurrence(i);
            SearchTokenOccurrence tokenOccurrence2 = list2.getSearchTokenOccurrence(i);
            int tk1 = tokenOccurrence1.getOperandNumber();
            int tk2 = tokenOccurrence2.getOperandNumber();
            if (tk1 < tk2) {
                return 1;
            }
            if (tk1 > tk1) {
                return -1;
            }
            int start1 = tokenOccurrence1.getBeginIndex();
            int start2 = tokenOccurrence2.getBeginIndex();
            if (start1 < start2) {
                return 1;
            }
            if (start1 > start2) {
                return -1;
            }
            if (diff == 0) {
                int ln1 = tokenOccurrence1.getLength();
                int ln2 = tokenOccurrence2.getLength();
                if (ln1 < ln2) {
                    diff = 1;
                }
                if (ln1 > ln2) {
                    diff = -1;
                }
            }
        }
        if (diff != 0) {
            return diff;
        }
        return 0;
    }

}
