/* ScrutariLib_SearchEngine - Copyright (c) 2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.searchengine.tools.result;

import net.mapeadores.util.text.SubstringPosition;
import net.mapeadores.util.text.collation.CollationUnit;
import net.mapeadores.util.text.collation.map.SearchResultUnit;
import net.scrutari.searchengine.api.result.SearchTokenOccurrenceList;
import net.scrutari.searchengine.api.result.SupermotcleSearchResultInfo;
import net.scrutari.supermotcle.api.Supermotcle;


/**
 *
 * @author Vincent Calame
 */
public class SupermotcleSearchResultInfoBuilder {

    private Supermotcle supermotcle;
    private SearchTokenOccurrenceListBuilder searchTokenPositionListBuilder;

    public SupermotcleSearchResultInfoBuilder(Supermotcle supermotcle, int operandMaxNumber) {
        this.supermotcle = supermotcle;
        this.searchTokenPositionListBuilder = new SearchTokenOccurrenceListBuilder(operandMaxNumber);
    }

    public void addSearchResultUnit(int operandNumber, SearchResultUnit sru) {
        CollationUnit collationUnit = supermotcle.getCollationUnit();
        SubstringPosition positionInSourceString = collationUnit.getSubstringPositionInSourceString(sru.getSearchTextPositionInCollatedKey());
        searchTokenPositionListBuilder.add(operandNumber, positionInSourceString);
    }

    public SupermotcleSearchResultInfo toSupermotcleSearchResultInfo() {
        return new InternalSupermotcleSearchResultInfo(supermotcle, searchTokenPositionListBuilder.toSearchTokenOccurrenceList());
    }


    private static class InternalSupermotcleSearchResultInfo implements SupermotcleSearchResultInfo {

        private SearchTokenOccurrenceList searchTokenPositionList;
        private Supermotcle supermotcle;

        private InternalSupermotcleSearchResultInfo(Supermotcle supermotcle, SearchTokenOccurrenceList searchTokenPositionList) {
            this.supermotcle = supermotcle;
            this.searchTokenPositionList = searchTokenPositionList;
        }

        public Supermotcle getSupermotcle() {
            return supermotcle;
        }

        public SearchTokenOccurrenceList getSearchTokentPositionList() {
            return searchTokenPositionList;
        }

    }

}
