/* SctServer - Copyright (c) 2015-2016 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.json;

import static fr.exemole.sctserver.json.JsonUtils.writeMLib;
import fr.exemole.sctserver.request.MotcleFieldsFilter;
import java.io.IOException;
import java.util.Set;
import net.mapeadores.util.json.JSONWriter;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.text.Label;
import net.scrutari.data.MotcleData;
import net.scrutari.datauri.MotcleURI;
import net.scrutari.searchengine.api.result.InLangOccurrenceList;
import net.scrutari.searchengine.api.result.MotcleSearchResultInfo;


/**
 *
 * @author Vincent Calame
 */
public final class MotcleDataJson {

    private MotcleDataJson() {

    }

    public static void object(JSONWriter jsonWriter, MotcleData motcleData, MotcleFieldsFilter motcleFieldsFilter, MotcleSearchResultInfo motcleSearchResultInfo, Lang langUi) throws IOException {
        jsonWriter.key("codemotcle");
        jsonWriter.value(motcleData.getCodeMotcle());
        if (motcleFieldsFilter.isWithCodeThesaurus()) {
            jsonWriter.key("codethesaurus");
            jsonWriter.value(motcleData.getCodeThesaurus());
        }
        if (motcleFieldsFilter.isWithCodeBase()) {
            jsonWriter.key("codebase");
            jsonWriter.value(motcleData.getCodeBase());
        }
        if (motcleFieldsFilter.isWithBaseName()) {
            MotcleURI motcleURI = motcleData.getMotcleURI();
            jsonWriter.key("basename");
            jsonWriter.value(motcleURI.getThesaurusURI().getBaseURI().getBaseName());
        }
        if (motcleFieldsFilter.isWithThesaurusName()) {
            MotcleURI motcleURI = motcleData.getMotcleURI();
            jsonWriter.key("thesaurusname");
            jsonWriter.value(motcleURI.getThesaurusURI().getThesaurusName());
        }
        if (motcleFieldsFilter.isWithMotcleId()) {
            MotcleURI motcleURI = motcleData.getMotcleURI();
            jsonWriter.key("motcleid");
            jsonWriter.value(motcleURI.getMotcleId());
        }
        if (motcleFieldsFilter.isWithLibelles()) {
            int libcount = motcleData.getLabelCount();
            if (libcount > 0) {
                jsonWriter.key("libelleArray");
                jsonWriter.array();
                for (int i = 0; i < libcount; i++) {
                    jsonWriter.object();
                    Label lib = motcleData.getLabelByIndex(i);
                    jsonWriter.key("lang");
                    jsonWriter.value(lib.getLang().toString());
                    jsonWriter.key("lib");
                    jsonWriter.value(lib.getLabelString());
                    jsonWriter.endObject();
                }
                jsonWriter.endArray();
            }
        }
        if (motcleFieldsFilter.isWithMLibelles()) {
            boolean withDefault = motcleFieldsFilter.isWithLibellesDefault();
            if (withDefault) {
                addDefaultMLibelles(jsonWriter, motcleData, motcleSearchResultInfo, true);
            } else {
                addMLibelles(jsonWriter, motcleData, motcleSearchResultInfo, langUi);
            }
        }
    }

    private static void addDefaultMLibelles(JSONWriter jsonWriter, MotcleData motcleData, MotcleSearchResultInfo motcleSearchResultInfo, boolean withDefault) throws IOException {
        int libcount = motcleData.getLabelCount();
        boolean inclus = false;
        for (int i = 0; i < libcount; i++) {
            Label lib = motcleData.getLabelByIndex(i);
            Lang lang = lib.getLang();
            InLangOccurrenceList occurrenceList = motcleSearchResultInfo.getInLangOccurrenceListByLang(lang);
            if ((!withDefault) && (occurrenceList == null)) {
                continue;
            }
            if (!inclus) {
                jsonWriter.key("mlibelleArray");
                jsonWriter.array();
                inclus = true;
            }
            jsonWriter.object();
            jsonWriter.key("lang");
            jsonWriter.value(lang.toString());
            jsonWriter.key("mlib");
            writeMLib(jsonWriter, lib.getLabelString(), occurrenceList);
            jsonWriter.endObject();
        }
        if (inclus) {
            jsonWriter.endArray();
        }
    }

    private static void addMLibelles(JSONWriter jsonWriter, MotcleData motcleData, MotcleSearchResultInfo motcleSearchResultInfo, Lang langUi) throws IOException {
        InLangOccurrenceList uiOccurrenceList = motcleSearchResultInfo.getInLangOccurrenceListByLang(langUi);
        if (uiOccurrenceList == null) {
            addDefaultMLibelles(jsonWriter, motcleData, motcleSearchResultInfo, false);
            return;
        }
        jsonWriter.key("mlibelleArray");
        jsonWriter.array();
        Label uiLib = motcleData.getLabelByLang(langUi);
        jsonWriter.object();
        jsonWriter.key("lang");
        jsonWriter.value(langUi.toString());
        jsonWriter.key("mlib");
        writeMLib(jsonWriter, uiLib.getLabelString(), uiOccurrenceList);
        jsonWriter.endObject();
        int libCount = motcleData.getLabelCount();
        if (libCount > 1) {
            Set<Integer> operandeSet = JsonUtils.getOperandSet(uiOccurrenceList);
            for (int i = 0; i < libCount; i++) {
                Label lib = motcleData.getLabelByIndex(i);
                Lang lang = lib.getLang();
                if (lang.equals(langUi)) {
                    continue;
                }
                InLangOccurrenceList occurrenceList = motcleSearchResultInfo.getInLangOccurrenceListByLang(lang);
                if (occurrenceList == null) {
                    continue;
                }
                if (JsonUtils.isCovered(occurrenceList, operandeSet)) {
                    continue;
                }
                jsonWriter.object();
                jsonWriter.key("lang");
                jsonWriter.value(lang.toString());
                jsonWriter.key("mlib");
                writeMLib(jsonWriter, lib.getLabelString(), occurrenceList);
                jsonWriter.endObject();
            }
        }
        jsonWriter.endArray();
    }

}
