/* SctServer - Copyright (c) 2011-2016 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.request.json;

import fr.exemole.sctserver.json.CategoryJson;
import java.io.IOException;
import net.mapeadores.util.json.JSONWriter;
import net.mapeadores.util.localisation.Lang;
import net.scrutari.searchengine.api.options.Category;
import net.scrutari.searchengine.api.options.GlobalSearchOptions;


/**
 *
 * @author Vincent Calame
 */
public class CategoryJsonProducer extends AbstractJsonProducer {

    private final int version;
    private final Lang langUi;
    private final GlobalSearchOptions globalSearchOptions;

    public CategoryJsonProducer(int version, GlobalSearchOptions globalSearchOptions, Lang langUi) {
        if (langUi == null) {
            throw new IllegalArgumentException("langUi is null");
        }
        this.version = version;
        this.globalSearchOptions = globalSearchOptions;
        this.langUi = langUi;
    }

    @Override
    public void writeJson(Appendable appendable) throws IOException {
        JSONWriter jsonWriter = new JSONWriter(appendable);
        jsonWriter.object();
        if (version > 1) {
            printVersion2(jsonWriter);
        } else {
            printVersion1(jsonWriter);
        }
        writeWarnings(jsonWriter);
        jsonWriter.endObject();
    }

    private void printVersion2(JSONWriter jsonWriter) throws IOException {
        int categoryCount = globalSearchOptions.getCategoryCount();
        jsonWriter.key("categoryArray");
        jsonWriter.array();
        for (int i = 0; i < categoryCount; i++) {
            Category category = globalSearchOptions.getCategoryByIndex(i);
            CategoryJson.object(jsonWriter, category, langUi, true);
        }
        jsonWriter.endArray();
    }

    private void printVersion1(JSONWriter jsonWriter) throws IOException {
        int categoryCount = globalSearchOptions.getCategoryCount();
        jsonWriter.key("categoryData");
        jsonWriter.object();
        jsonWriter.key("categoryCount");
        jsonWriter.value(categoryCount);
        if (categoryCount > 0) {
            jsonWriter.key("categoryArray");
            jsonWriter.array();
            for (int i = 0; i < categoryCount; i++) {
                Category category = globalSearchOptions.getCategoryByIndex(i);
                CategoryJson.object(jsonWriter, category, langUi, true);
            }
            jsonWriter.endArray();
        }
        jsonWriter.endObject();
    }


}
