/* Scrutari - Copyright (c) 2016 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.request.json;

import fr.exemole.sctserver.api.EngineContext;
import fr.exemole.sctserver.api.SctEngine;
import fr.exemole.sctserver.api.context.EngineGroup;
import fr.exemole.sctserver.json.JsonUtils;
import java.io.IOException;
import net.mapeadores.util.attr.AttributeHolder;
import net.mapeadores.util.json.CommonJson;
import net.mapeadores.util.json.JSONWriter;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.text.LabelHolder;


/**
 *
 * @author Vincent Calame
 */
public class EngineGroupJsonProducer extends AbstractJsonProducer {

    private final int version;
    private final EngineContext engineContext;
    private final EngineGroup engineGroup;
    private final Lang langUi;
    private final boolean withLabelMap;


    public EngineGroupJsonProducer(int version, EngineContext engineContext, EngineGroup engineGroup, Lang langUi, boolean withLabelMap) {
        if (langUi == null) {
            throw new IllegalArgumentException("langUi is null");
        }
        this.version = version;
        this.engineContext = engineContext;
        this.engineGroup = engineGroup;
        this.langUi = langUi;
        this.withLabelMap = withLabelMap;
    }

    @Override
    public void writeJson(Appendable appendable) throws IOException {
        String url = engineContext.getWebappUrl();
        JSONWriter jsonWriter = new JSONWriter(appendable);
        jsonWriter.object();
        jsonWriter.key("engineGroup");
        jsonWriter.object();
        jsonWriter.key("name");
        jsonWriter.value(engineGroup.getName());
        CommonJson.defaultLabel(jsonWriter, engineGroup, langUi, "title", engineGroup.getName());
        if (withLabelMap) {
            jsonWriter.key("labelMap");
            CommonJson.object(jsonWriter, (LabelHolder) engineGroup);
        }
        jsonWriter.key("attrMap");
        CommonJson.object(jsonWriter, (AttributeHolder) engineGroup);
        int engineCount = engineGroup.getEngineCount();
        jsonWriter.key("engineArray");
        jsonWriter.array();
        for (int i = 0; i < engineCount; i++) {
            String engineName = engineGroup.getEngineName(i);
            jsonWriter.object();
            jsonWriter.key("name");
            jsonWriter.value(engineName);
            jsonWriter.key("url");
            jsonWriter.value(url + engineName + "/");
            SctEngine engine = engineContext.getEngine(engineName);
            if (engine != null) {
                JsonUtils.addMetadataProperties(jsonWriter, engineName, engine.getEngineMetadata(), langUi, withLabelMap);
            }
            jsonWriter.endObject();
        }
        jsonWriter.endArray();
        jsonWriter.endObject();
        writeWarnings(jsonWriter);
        jsonWriter.endObject();
    }

}
