/* SctServer - Copyright (c) 2012-2016 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.tools.conf.factory;

import fr.exemole.sctserver.tools.ErrorUtils;
import fr.exemole.sctserver.tools.conf.dom.AttributesDOMReader;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import net.mapeadores.util.attr.AttributeDef;
import net.mapeadores.util.attr.AttributeDefList;
import net.mapeadores.util.attr.AttributeKey;
import net.mapeadores.util.attr.AttributeUtils;
import net.mapeadores.util.logging.CategorizedMessageList;
import net.mapeadores.util.logging.SimpleMessageHandler;
import net.scrutari.db.api.AttributeKeyCoder;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;


/**
 *
 * @author Vincent Calame
 */
final class AttributesConf {

    private String processURI;
    private CategorizedMessageList messageList;
    private Map<String, AttributeDefList> attributeDefListMap;

    private AttributesConf() {
    }

    Map<String, AttributeDefList> getAttributeDefListMap() {
        return attributeDefListMap;
    }

    String getProcessURI() {
        return processURI;
    }

    CategorizedMessageList getMessageList() {
        return messageList;
    }

    static AttributesConf load(File f) throws IOException, ParserConfigurationException {
        AttributesConf conf = new AttributesConf();
        conf.init(f);
        return conf;
    }

    private void init(File f) throws IOException, ParserConfigurationException {
        processURI = f.getPath();
        SimpleMessageHandler simpleHandler = new SimpleMessageHandler();
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        Document document;
        try {
            DocumentBuilder db = dbf.newDocumentBuilder();
            document = db.parse(f);
        } catch (SAXException saxe) {
            ErrorUtils.xml(simpleHandler, "_ error.exception.xml.sax", saxe.getMessage());
            messageList = simpleHandler.toList();
            return;
        }
        AttributesBuilder builder = new AttributesBuilder();
        AttributesDOMReader domReader = new AttributesDOMReader(builder, simpleHandler);
        domReader.readAttributes(document.getDocumentElement(), "");
        attributeDefListMap = builder.toMap();
        messageList = simpleHandler.toList();
    }


    private static class AttributesBuilder implements AttributesDOMReader.Builder {

        private final Set<AttributeKey> keySet = new HashSet<AttributeKey>();
        private final Map<String, List<AttributeDef>> listMap = new HashMap<String, List<AttributeDef>>();
        private List<AttributeDef> currentList;

        private AttributesBuilder() {

        }

        @Override
        public boolean startGroup(String name) {
            if (isValidGroupName(name)) {
                List<AttributeDef> list = listMap.get(name);
                if (list == null) {
                    list = new ArrayList<AttributeDef>();
                    listMap.put(name, list);
                }
                currentList = list;
                return true;
            } else {
                return false;
            }
        }

        @Override
        public boolean addAttributeDef(AttributeDef attributeDef) {
            AttributeKey attributeKey = attributeDef.getAttributeKey();
            if (keySet.contains(attributeKey)) {
                return false;
            }
            if (currentList == null) {
                throw new IllegalStateException("startGroup must be successfully invoked first");
            }
            currentList.add(attributeDef);
            keySet.add(attributeKey);
            return true;
        }

        private Map<String, AttributeDefList> toMap() {
            Map<String, AttributeDefList> map = new HashMap<String, AttributeDefList>();
            for (Map.Entry<String, List<AttributeDef>> entry : listMap.entrySet()) {
                map.put(entry.getKey(), AttributeUtils.toAttributeDefList(entry.getValue()));
            }
            return map;
        }

    }

    private static boolean isValidGroupName(String name) {
        switch (name) {
            case AttributeKeyCoder.PRIMARY_GROUP:
            case AttributeKeyCoder.SECONDARY_GROUP:
            case AttributeKeyCoder.TECHNICAL_GROUP:
                return true;
            default:
                return false;

        }
    }

}
