/* SctServer - Copyright (c) 2011-2016 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.tools.conf.factory;

import fr.exemole.sctserver.tools.ErrorUtils;
import fr.exemole.sctserver.tools.conf.dom.UriCodesDOMReader;
import java.io.File;
import java.io.IOException;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import net.mapeadores.util.logging.CategorizedMessageList;
import net.mapeadores.util.logging.SimpleMessageHandler;
import net.scrutari.datauri.ScrutariDataURI;
import net.scrutari.db.api.codes.ScrutariDataURICodeList;
import net.scrutari.db.tools.codes.ScrutariDataCodeListBuilder;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;


/**
 *
 * @author Vincent Calame
 */
final class UriCodesConf {

    private ScrutariDataURICodeList scrutariDataURICodeList;
    private String processURI;
    private CategorizedMessageList messageList;

    private UriCodesConf() {
    }

    ScrutariDataURICodeList getScrutariDataURICodeList() {
        return scrutariDataURICodeList;
    }

    static UriCodesConf load(File f) throws IOException, ParserConfigurationException {
        UriCodesConf conf = new UriCodesConf();
        conf.init(f);
        return conf;
    }

    String getProcessURI() {
        return processURI;
    }

    CategorizedMessageList getMessageList() {
        return messageList;
    }

    private void init(File f) throws IOException, ParserConfigurationException {
        processURI = f.getPath();
        SimpleMessageHandler simpleHandler = new SimpleMessageHandler();
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        Document document;
        try {
            DocumentBuilder db = dbf.newDocumentBuilder();
            document = db.parse(f);
        } catch (SAXException saxe) {
            ErrorUtils.xml(simpleHandler, "_ error.exception.xml.sax", saxe.getMessage());
            messageList = simpleHandler.toList();
            return;
        }
        UriCodesBuilder builder = new UriCodesBuilder();
        UriCodesDOMReader domReader = new UriCodesDOMReader(builder, simpleHandler);
        domReader.readUriCodes(document.getDocumentElement(), "");
        scrutariDataURICodeList = builder.toScrutariDataURICodeList();
        messageList = simpleHandler.toList();
    }


    private static class UriCodesBuilder implements UriCodesDOMReader.Builder {

        private final ScrutariDataCodeListBuilder scrutariDataCodeListBuilder = new ScrutariDataCodeListBuilder();


        public UriCodesBuilder() {

        }

        @Override
        public boolean containsCode(int code) {
            return scrutariDataCodeListBuilder.containsCode(code);
        }

        @Override
        public boolean containsURI(ScrutariDataURI scrutariDataURI) {
            return scrutariDataCodeListBuilder.containsURI(scrutariDataURI);
        }

        @Override
        public void addScrutariDataURI(int code, ScrutariDataURI scrutariDataURI) {
            scrutariDataCodeListBuilder.addScrutariDataURI(code, scrutariDataURI);
        }

        public ScrutariDataURICodeList toScrutariDataURICodeList() {
            if (scrutariDataCodeListBuilder.size() == 0) {
                return null;
            } else {
                return scrutariDataCodeListBuilder.toScrutariDataURICodeList();
            }
        }

    }


}
