/* SctServer - Copyright (c) 2016 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.tools.context;

import fr.exemole.sctserver.api.context.EngineGroup;
import java.util.LinkedHashSet;
import java.util.Set;
import net.mapeadores.util.attr.AttributeHolder;
import net.mapeadores.util.misc.DefBuilder;
import net.mapeadores.util.text.LabelHolder;


/**
 *
 * @author Vincent Calame
 */
public class EngineGroupBuilder extends DefBuilder {

    private final String name;
    private final Set<String> engineNameSet = new LinkedHashSet();

    public EngineGroupBuilder(String name) {
        this.name = name;
    }

    public void addEngine(String engineName) {
        if (engineName == null) {
            throw new IllegalArgumentException("engineName is null");
        }
        engineNameSet.add(engineName);
    }

    public EngineGroup toEngineGroup() {
        String[] engineNameArray = engineNameSet.toArray(new String[engineNameSet.size()]);
        LabelHolder labelHolder = toLabelHolder();
        AttributeHolder attributeHolder = toAttributeHolder();
        return new InternalEngineGroup(name, engineNameArray, labelHolder, attributeHolder);
    }


    private static class InternalEngineGroup extends DefBuilder.Def implements EngineGroup {

        private final String name;
        private final String[] engineNameArray;

        private InternalEngineGroup(String name, String[] engineNameArray,
                LabelHolder labelHolder, AttributeHolder attributeHolder) {
            super(labelHolder, attributeHolder);
            this.name = name;
            this.engineNameArray = engineNameArray;
        }

        @Override
        public String getName() {
            return name;
        }

        @Override
        public int getEngineCount() {
            return engineNameArray.length;
        }

        @Override
        public String getEngineName(int index) {
            return engineNameArray[index];
        }

    }

}
