/* UtilLib - Copyright (c) 2011-2016 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.attr;

import java.util.LinkedHashSet;
import java.util.Set;
import net.mapeadores.util.text.ChangeLabelHolderBuilder;
import net.mapeadores.util.text.Label;
import net.mapeadores.util.text.LabelHolder;


/**
 *
 * @author Vincent Calame
 */
public class AttributeDefBuilder {

    private final Set<String> scopeSet = new LinkedHashSet<String>();
    private final AttributeKey attributeKey;
    private String formatType = "";
    private Object formatObject;
    private ChangeLabelHolderBuilder labelHolderBuilder;

    public AttributeDefBuilder(AttributeKey attributeKey) {
        if (attributeKey == null) {
            throw new IllegalArgumentException("attributeKey is null");
        }
        this.attributeKey = attributeKey;
        labelHolderBuilder = new ChangeLabelHolderBuilder();
    }

    public AttributeKey getAttributeKey() {
        return attributeKey;
    }

    public AttributeDef toAttributeDef() {
        String[] scopeArray = scopeSet.toArray(new String[scopeSet.size()]);
        LabelHolder labelHolder = labelHolderBuilder.toChangeLabelHolder();
        return new InternalAttributeDef(attributeKey, scopeArray, formatType, formatObject, labelHolder);
    }

    public void addScope(String s) {
        scopeSet.add(s);
    }

    public void setFormatType(String formatType) {
        if (formatType == null) {
            this.formatType = "";
        } else {
            this.formatType = formatType;
        }
    }

    public void addLabel(Label label) {
        labelHolderBuilder.putLabel(label);
    }

    public static AttributeDefBuilder derive(AttributeDef attributeDef) {
        AttributeDefBuilder builder = new AttributeDefBuilder(attributeDef.getAttributeKey());
        int scopeCount = attributeDef.getScopeCount();
        for (int i = 0; i < scopeCount; i++) {
            builder.addScope(attributeDef.getScope(i));
        }
        builder.setFormatType(attributeDef.getFormatType());
        LabelHolder labelHolder = attributeDef.getLabelHolder();
        int labelCount = labelHolder.getLabelCount();
        for (int i = 0; i < labelCount; i++) {
            builder.addLabel(labelHolder.getLabelByIndex(i));
        }
        return builder;
    }


    private static class InternalAttributeDef implements AttributeDef {

        private final AttributeKey attributeKey;
        private final String[] scopeArray;
        private final String formatType;
        private final Object formatObject;
        private final LabelHolder labelHolder;

        private InternalAttributeDef(AttributeKey attributeKey, String[] scopeArray, String formatType, Object formatObject, LabelHolder labelHolder) {
            this.attributeKey = attributeKey;
            this.scopeArray = scopeArray;
            this.formatType = formatType;
            this.formatObject = formatObject;
            this.labelHolder = labelHolder;
        }

        @Override
        public AttributeKey getAttributeKey() {
            return attributeKey;
        }

        @Override
        public int getScopeCount() {
            return scopeArray.length;
        }

        @Override
        public String getScope(int i) {
            return scopeArray[i];
        }

        @Override
        public String getFormatType() {
            return formatType;
        }

        @Override
        public Object getFormatObject() {
            return formatObject;
        }

        @Override
        public LabelHolder getLabelHolder() {
            return labelHolder;
        }

    }

}
