/* Util_Lib - Copyright (c) 2010-2016 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.buildinfo;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.text.ParseException;
import net.mapeadores.util.primitives.FuzzyDate;


/**
 *
 * @author Vincent Calame
 */
public class BuildInfoParser {

    public static BuildInfo parse(InputStream inputStream) throws IOException {
        BufferedReader reader = new BufferedReader(new InputStreamReader(inputStream, "UTF-8"));
        InternalBuildInfo internalBuildInfo = new InternalBuildInfo();
        String ligne;
        while ((ligne = reader.readLine()) != null) {
            internalBuildInfo.parse(ligne);
        }
        return internalBuildInfo;
    }


    private static class InternalBuildInfo implements BuildInfo {

        private String softwareVersion = "";
        private int revisionNumber = 0;
        private FuzzyDate date = null;

        @Override
        public String getSoftwareVersion() {
            return softwareVersion;
        }

        @Override
        public int getRevisionNumber() {
            return revisionNumber;
        }

        @Override
        public FuzzyDate getDate() {
            return date;
        }

        private void parse(String s) {
            int idx = s.indexOf(':');
            if (idx == -1) {
                return;
            }
            String key = s.substring(0, idx).trim();
            if (key.length() == 0) {
                return;
            }
            key = key.toLowerCase();
            String value = s.substring(idx + 1).trim();
            if (value.length() == 0) {
                return;
            }
            if (key.equals("version")) {
                softwareVersion = value;
            } else if (key.equals("svn-revision")) {
                try {
                    revisionNumber = Integer.parseInt(value);
                } catch (NumberFormatException nfe) {
                }
            } else if (key.equals("date")) {
                try {
                    date = FuzzyDate.parse(value);
                } catch (ParseException pe) {
                }
            }
        }

    }

}
