/* UtilLib - Copyright (c) 2016 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.conditions;

import java.util.ArrayList;
import java.util.List;


/**
 *
 * @author Vincent Calame
 */
public class ConditionBuilder {

    private final short mode;
    private final List<TextTest> excludingList = new ArrayList<TextTest>();
    private final List<TextTest> includingList = new ArrayList<TextTest>();
    private short state = ConditionsConstants.UNKNOWN_STATE;

    public ConditionBuilder(short mode) {
        this.mode = mode;
    }

    public void setState(short state) {
        this.state = state;
    }

    public void addTextTest(TextTest textTest) {
        short testType = textTest.getTestType();
        short addingState = ConditionsUtils.getState(testType);
        state = ConditionsUtils.merge(mode, state, addingState);
        if (!ConditionsUtils.isPartialState(state)) {
            excludingList.clear();
            includingList.clear();
        } else {
            if (ConditionsUtils.isExcludingPartTestType(testType)) {
                if (excludingList.size() < 31) {
                    excludingList.add(ConditionsUtils.cloneTextTest(textTest));
                }
            } else if (ConditionsUtils.isIncludingPartTestType(testType)) {
                if (includingList.size() < 31) {
                    includingList.add(ConditionsUtils.cloneTextTest(textTest));
                }
            }
        }
    }

    public Condition toCondition() {
        TextTest[] excludingArray = excludingList.toArray(new TextTest[excludingList.size()]);
        TextTest[] includingArray = includingList.toArray(new TextTest[includingList.size()]);
        return new InternalCondition(mode, state, excludingArray, includingArray);
    }

    public static Condition build(TextTest textTest) {
        ConditionBuilder conditionBuilder = new ConditionBuilder(ConditionsConstants.MODE_AND);
        conditionBuilder.addTextTest(textTest);
        return conditionBuilder.toCondition();
    }


    private static class InternalCondition implements Condition {

        private final short mode;
        private final short state;
        private final TextTest[] excludingArray;
        private final TextTest[] includingArray;

        public InternalCondition(short mode, short state, TextTest[] excludingArray, TextTest[] includingArray) {
            this.mode = mode;
            this.state = state;
            this.excludingArray = excludingArray;
            this.includingArray = includingArray;
        }

        @Override
        public short getMode() {
            return mode;
        }

        @Override
        public short getState() {
            return state;
        }

        @Override
        public int getExcludingTextTestCount() {
            return excludingArray.length;
        }

        @Override
        public int getIncludingTextTestCount() {
            return includingArray.length;
        }

        @Override
        public TextTest getExcludingTextTest(int i) {
            return excludingArray[i];
        }

        @Override
        public TextTest getIncludingTextTest(int i) {
            return includingArray[i];
        }

    }

}
