/* UtilLib - Copyright (c) 2008-2016 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.logging;

import net.mapeadores.util.localisation.Message;
import net.mapeadores.util.localisation.MessageList;


/**
 *
 * @author Vincent Calame
 */
public class CommandMessage implements Message {

    private final static Object[] EMPTY = new Object[0];
    private String messageKey;
    private Object[] messageValues;
    private MessageLog messageLog;
    private MessageList multiErrorList;
    private boolean errorMessage = false;

    private CommandMessage(String messageKey, Object[] messageValues) {
        if (messageKey == null) {
            throw new IllegalArgumentException("messageKey is null");
        }
        if (messageValues == null) {
            messageValues = EMPTY;
        }
        this.messageKey = messageKey;
        this.messageValues = messageValues;
    }

    private CommandMessage(Message message) {
        this.messageKey = message.getMessageKey();
        Object[] values = message.getMessageValues();
        if (values == null) {
            values = EMPTY;
        }
        this.messageValues = values;
    }

    private void initIsErrorMessage(boolean errorMessage) {
        this.errorMessage = errorMessage;
    }

    private void initMultiErrorList(MessageList multiErrorList) {
        this.multiErrorList = multiErrorList;
    }

    private void initMessageLog(MessageLog messageLog) {
        this.messageLog = messageLog;
    }

    @Override
    public String getMessageKey() {
        return messageKey;
    }

    @Override
    public Object[] getMessageValues() {
        return messageValues;
    }

    public boolean isErrorMessage() {
        return errorMessage;
    }

    public boolean hasMultiError() {
        return (multiErrorList != null);
    }

    public MessageList getMultiErrorList() {
        return multiErrorList;
    }

    public boolean hasLog() {
        return ((messageLog != null) && (messageLog.getLogGroupCount() > 0));
    }

    public MessageLog getMessageLog() {
        return messageLog;
    }

    public static CommandMessage error(String messageKey, Object... messageValues) {
        CommandMessage commandMessage = new CommandMessage(messageKey, messageValues);
        commandMessage.initIsErrorMessage(true);
        return commandMessage;
    }

    public static CommandMessage error(Message message) {
        CommandMessage commandMessage = new CommandMessage(message);
        commandMessage.initIsErrorMessage(true);
        return commandMessage;
    }

    public static CommandMessage error(Message message, MessageList multiErrorList) {
        CommandMessage commandMessage = new CommandMessage(message);
        commandMessage.initIsErrorMessage(true);
        commandMessage.initMultiErrorList(multiErrorList);
        return commandMessage;
    }

    public static CommandMessage done(String messageKey, Object... messageValues) {
        CommandMessage commandMessage = new CommandMessage(messageKey, messageValues);
        return commandMessage;
    }

    public static CommandMessage done(Message message) {
        CommandMessage commandMessage = new CommandMessage(message);
        return commandMessage;
    }

    public static CommandMessage done(Message message, MessageLog processMessageLog) {
        CommandMessage commandMessage = new CommandMessage(message);
        commandMessage.initMessageLog(processMessageLog);
        return commandMessage;
    }

}
