/* UtilLib - Copyright (c) 2010-2016 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.logging;

import java.io.IOException;
import net.mapeadores.util.json.CommonJson;
import net.mapeadores.util.json.JSONWriter;
import net.mapeadores.util.json.JsonProducer;
import net.mapeadores.util.localisation.MessageLocalisation;


/**
 *
 * @author Vincent Calame
 */
public class CommandMessageJsonProducer implements JsonProducer {

    private CommandMessage[] commandMessages;
    private MessageLocalisation messageLocalisation;
    private boolean errorMessage = false;
    private String messageKey;
    private String messageText;

    public CommandMessageJsonProducer(MessageLocalisation messageLocalisation, CommandMessage... commandMessages) {
        if (commandMessages.length == 0) {
            throw new IllegalArgumentException("commandMessages is empty");
        }
        this.commandMessages = commandMessages;
        this.messageLocalisation = messageLocalisation;
    }

    public CommandMessageJsonProducer(String messageKey, String messageText, boolean errorMessage) {
        this.errorMessage = errorMessage;
        this.messageKey = messageKey;
        this.messageText = messageText;
    }

    @Override
    public void writeJson(Appendable appendable) throws IOException {
        JSONWriter jsonWriter = new JSONWriter(appendable);
        jsonWriter.object();
        if (commandMessages != null) {
            jsonWriter.key("commandMessage");
            CommonJson.object(jsonWriter, commandMessages[commandMessages.length - 1], messageLocalisation);
            jsonWriter.key("commandMessageArray");
            jsonWriter.array();
            for (CommandMessage commandMessage : commandMessages) {
                CommonJson.object(jsonWriter, commandMessage, messageLocalisation);
            }
            jsonWriter.endArray();
        } else {
            jsonWriter.key("commandMessage");
            writeSimple(jsonWriter);
        }
        jsonWriter.endObject();
    }

    private void writeSimple(JSONWriter jsonWriter) throws IOException {
        jsonWriter.object();
        jsonWriter.key("type");
        if (errorMessage) {
            jsonWriter.value("error");
        } else {
            jsonWriter.value("done");
        }
        jsonWriter.key("key");
        jsonWriter.value(messageKey);
        if (messageText != null) {
            jsonWriter.key("text");
            jsonWriter.value(messageText);
        }
        jsonWriter.endObject();
    }

}
