/* UtilLib - Copyright (c) 2016 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.logging;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;


/**
 *
 * @author Vincent Calame
 */
public class MessageByLineLogBuilder {

    private final Map<String, SimpleLineMessageHandler> handlerMap = new LinkedHashMap<String, SimpleLineMessageHandler>();


    public LineMessageHandler setCurrentURI(String uri) {
        SimpleLineMessageHandler handler = handlerMap.get(uri);
        if (handler == null) {
            handler = new SimpleLineMessageHandler();
            handlerMap.put(uri, handler);
        }
        return handler;
    }

    public MessageByLineLog toMessageByLineLog() {
        List<InternalLogGroup> logGroupList = new ArrayList<InternalLogGroup>();
        for (Map.Entry<String, SimpleLineMessageHandler> entry : handlerMap.entrySet()) {
            SimpleLineMessageHandler handler = entry.getValue();
            if (handler.hasMessage()) {
                MessageByLine[] array = handler.getMessageByLineList().toArray(new MessageByLine[1]);
                logGroupList.add(new InternalLogGroup(entry.getKey(), array));
            }
        }
        InternalLogGroup[] array = logGroupList.toArray(new InternalLogGroup[logGroupList.size()]);
        return new InternalMessageByLineLog(array);
    }


    private static class InternalMessageByLineLog implements MessageByLineLog {

        private final InternalLogGroup[] array;

        private InternalMessageByLineLog(InternalLogGroup[] array) {
            this.array = array;
        }

        @Override
        public int getLogGroupCount() {
            return array.length;
        }

        @Override
        public LogGroup getLogGroup(int i) {
            return array[i];
        }

    }


    private static class InternalLogGroup implements MessageByLineLog.LogGroup {

        private final String uri;
        private final MessageByLine[] array;

        private InternalLogGroup(String uri, MessageByLine[] array) {
            this.uri = uri;
            this.array = array;
        }

        @Override
        public String getURI() {
            return uri;
        }

        @Override
        public int getMessageByLineCount() {
            return array.length;
        }

        @Override
        public MessageByLine getMessageByLine(int i) {
            return array[i];
        }

    }

}
