/* BdfServer - Copyright (c) 2012-2016 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.logging;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;
import net.mapeadores.util.localisation.Message;
import net.mapeadores.util.localisation.MessageList;


/**
 *
 * @author Vincent Calame
 */
public class SimpleLineMessageHandler implements LineMessageHandler {

    private final SortedMap<Integer, InternalMessageByLine> messageByLineMap = new TreeMap<Integer, InternalMessageByLine>();

    public SimpleLineMessageHandler() {
    }

    @Override
    public void addMessage(int lineNumber, String category, Message message) {
        InternalMessageByLine messageByLine = messageByLineMap.get(lineNumber);
        if (messageByLine == null) {
            messageByLine = new InternalMessageByLine(lineNumber);
            messageByLineMap.put(lineNumber, messageByLine);
        }
        messageByLine.add(category, message);
    }

    public boolean hasMessage() {
        return (!messageByLineMap.isEmpty());
    }

    public List<MessageByLine> getMessageByLineList() {
        ArrayList<MessageByLine> messageByLineList = new ArrayList<MessageByLine>();
        messageByLineList.addAll(messageByLineMap.values());
        return messageByLineList;
    }


    private static class InternalMessageByLine implements MessageByLine {

        private final int lineNumber;
        private final Map<String, Category> categoryMap = new HashMap<String, Category>();
        private final List<Category> categoryList = new ArrayList<Category>();

        private InternalMessageByLine(int lineNumber) {
            this.lineNumber = lineNumber;
        }

        @Override
        public int getLineNumber() {
            return lineNumber;
        }

        @Override
        public int getCategoryCount() {
            return categoryList.size();
        }

        @Override
        public String getCategory(int i) {
            return categoryList.get(i).getName();
        }

        @Override
        public MessageList getMessageList(int i) {
            return categoryList.get(i).getMessageList();
        }


        private void add(String categoryName, Message message) {
            Category category = categoryMap.get(categoryName);
            if (category == null) {
                category = new Category(categoryName);
                categoryMap.put(categoryName, category);
                categoryList.add(category);
            }
            category.add(message);
        }

    }


    private static class Category {

        private final String name;
        private final InternalMessageList messageList = new InternalMessageList();

        private Category(String name) {
            this.name = name;
        }

        private String getName() {
            return name;
        }

        private MessageList getMessageList() {
            return messageList;
        }

        private void add(Message message) {
            messageList.add(message);
        }

    }


    private static class InternalMessageList implements MessageList {

        private final List<Message> messageList = new ArrayList<Message>();

        private InternalMessageList() {

        }

        @Override
        public int getMessageCount() {
            return messageList.size();
        }

        @Override
        public Message getMessage(int i) {
            return messageList.get(i);
        }

        private void add(Message message) {
            messageList.add(message);
        }

    }

}
